@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -317,7 +316,7 @@ put_record_into_cache (ulong recno, const char *data)
 	}
 
       /* Now put into the cache.  */
-      assert (unused);
+      log_assert (unused);
       r = unused;
       r->flags.used = 1;
       r->recno = recno;
@@ -383,7 +382,7 @@ put_record_into_cache (ulong recno, const char *data)
       release_write_lock ();
 
       /* Now put into the cache.  */
-      assert (unused);
+      log_assert (unused);
       r = unused;
       r->flags.used = 1;
       r->recno = recno;
@@ -603,9 +602,10 @@ create_version_record (void)
 int
 tdbio_set_dbname (const char *new_dbname, int create, int *r_nofile)
 {
-  char *fname;
+  char *fname, *p;
   struct stat statbuf;
   static int initialized = 0;
+  int save_slash;
 
   if (!initialized)
     {
@@ -643,11 +643,48 @@ tdbio_set_dbname (const char *new_dbname, int create, int *r_nofile)
       /* OK, we have the valid trustdb.gpg already.  */
       return 0;
     }
+  else if (!create)
+    {
+      *r_nofile = 1;
+      return 0;
+    }
+
+  /* Here comes: No valid trustdb.gpg AND CREATE==1 */
+
+  /*
+   * Make sure the directory exists.  This should be done before
+   * acquiring the lock, which assumes the existence of the directory.
+   */
+  p = strrchr (fname, DIRSEP_C);
+#if HAVE_W32_SYSTEM
+  {
+    /* Windows may either have a slash or a backslash.  Take
+       care of it.  */
+    char *pp = strrchr (fname, '/');
+    if (!p || pp > p)
+      p = pp;
+  }
+#endif /*HAVE_W32_SYSTEM*/
+  log_assert (p);
+  save_slash = *p;
+  *p = 0;
+  if (access (fname, F_OK))
+    {
+      try_make_homedir (fname);
+      if (access (fname, F_OK))
+        log_fatal (_("%s: directory does not exist!\n"), fname);
+    }
+  *p = save_slash;
 
   take_write_lock ();
 
   if (access (fname, R_OK))
     {
+      FILE *fp;
+      TRUSTREC rec;
+      int rc;
+      mode_t oldmask;
+
 #ifdef HAVE_W32CE_SYSTEM
       /* We know how the cegcc implementation of access works ;-). */
       if (GetLastError () == ERROR_FILE_NOT_FOUND)
@@ -658,66 +695,34 @@ tdbio_set_dbname (const char *new_dbname, int create, int *r_nofile)
       if (errno != ENOENT)
         log_fatal ( _("can't access '%s': %s\n"), fname, strerror (errno));
 
-      if (!create)
-        *r_nofile = 1;
-      else
+      oldmask = umask (077);
+      if (is_secured_filename (fname))
         {
-          FILE *fp;
-          TRUSTREC rec;
-          int rc;
-          char *p = strrchr (fname, DIRSEP_C);
-          mode_t oldmask;
-          int save_slash;
+          fp = NULL;
+          gpg_err_set_errno (EPERM);
+        }
+      else
+        fp = fopen (fname, "wb");
+      umask(oldmask);
+      if (!fp)
+        log_fatal (_("can't create '%s': %s\n"), fname, strerror (errno));
+      fclose (fp);
 
-#if HAVE_W32_SYSTEM
-          {
-            /* Windows may either have a slash or a backslash.  Take
-               care of it.  */
-            char *pp = strrchr (fname, '/');
-            if (!p || pp > p)
-              p = pp;
-          }
-#endif /*HAVE_W32_SYSTEM*/
-          assert (p);
-          save_slash = *p;
-          *p = 0;
-          if (access (fname, F_OK))
-            {
-              try_make_homedir (fname);
-              if (access (fname, F_OK))
-                log_fatal (_("%s: directory does not exist!\n"), fname);
-	    }
-          *p = save_slash;
+      db_fd = open (db_name, O_RDWR | MY_O_BINARY);
+      if (db_fd == -1)
+        log_fatal (_("can't open '%s': %s\n"), db_name, strerror (errno));
 
-          oldmask = umask (077);
-          if (is_secured_filename (fname))
-            {
-              fp = NULL;
-              gpg_err_set_errno (EPERM);
-            }
-          else
-            fp = fopen (fname, "wb");
-          umask(oldmask);
-          if (!fp)
-            log_fatal (_("can't create '%s': %s\n"), fname, strerror (errno));
-          fclose (fp);
+      rc = create_version_record ();
+      if (rc)
+        log_fatal (_("%s: failed to create version record: %s"),
+                   fname, gpg_strerror (rc));
 
-          db_fd = open (db_name, O_RDWR | MY_O_BINARY);
-          if (db_fd == -1)
-            log_fatal (_("can't open '%s': %s\n"), db_name, strerror (errno));
+      /* Read again to check that we are okay. */
+      if (tdbio_read_record (0, &rec, RECTYPE_VER))
+        log_fatal (_("%s: invalid trustdb created\n"), db_name);
 
-          rc = create_version_record ();
-          if (rc)
-            log_fatal (_("%s: failed to create version record: %s"),
-                       fname, gpg_strerror (rc));
-
-          /* Read again to check that we are okay. */
-          if (tdbio_read_record (0, &rec, RECTYPE_VER))
-            log_fatal (_("%s: invalid trustdb created\n"), db_name);
-
-          if (!opt.quiet)
-            log_info (_("%s: trustdb created\n"), db_name);
-	}
+      if (!opt.quiet)
+        log_info (_("%s: trustdb created\n"), db_name);
     }
 
   release_write_lock ();
@@ -745,7 +750,7 @@ open_db ()
 {
   TRUSTREC rec;
 
-  assert( db_fd == -1 );
+  log_assert( db_fd == -1 );
 
 #ifdef HAVE_W32CE_SYSTEM
   {
@@ -805,7 +810,7 @@ create_hashtable( TRUSTREC *vr, int type )
   if (offset == -1)
     log_fatal ("trustdb: lseek to end failed: %s\n", strerror(errno));
   recnum = offset / TRUST_RECORD_LEN;
-  assert (recnum); /* This is will never be the first record. */
+  log_assert (recnum); /* This is will never be the first record. */
 
   if (!type)
     vr->r.ver.trusthashtbl = recnum;
@@ -1497,13 +1502,16 @@ tdbio_read_record (ulong recnum, TRUSTREC *rec, int expected)
           rec->r.ver.trust_model = *p++;
           rec->r.ver.min_cert_level = *p++;
           p += 2;
-          rec->r.ver.created  = buf32_to_ulong(p); p += 4;
-          rec->r.ver.nextcheck = buf32_to_ulong(p); p += 4;
+          rec->r.ver.created  = buf32_to_ulong(p);
+          p += 4;
+          rec->r.ver.nextcheck = buf32_to_ulong(p);
           p += 4;
           p += 4;
-          rec->r.ver.firstfree =buf32_to_ulong(p); p += 4;
           p += 4;
-          rec->r.ver.trusthashtbl =buf32_to_ulong(p); p += 4;
+          rec->r.ver.firstfree = buf32_to_ulong(p);
+          p += 4;
+          p += 4;
+          rec->r.ver.trusthashtbl = buf32_to_ulong(p);
           if (recnum)
             {
               log_error( _("%s: version record with recnum %lu\n"), db_name,
@@ -1520,37 +1528,43 @@ tdbio_read_record (ulong recnum, TRUSTREC *rec, int expected)
       break;
 
     case RECTYPE_FREE:
-      rec->r.free.next  = buf32_to_ulong(p); p += 4;
+      rec->r.free.next  = buf32_to_ulong(p);
       break;
 
     case RECTYPE_HTBL:
       for (i=0; i < ITEMS_PER_HTBL_RECORD; i++)
         {
-          rec->r.htbl.item[i] = buf32_to_ulong(p); p += 4;
+          rec->r.htbl.item[i] = buf32_to_ulong(p);
+          p += 4;
 	}
       break;
 
     case RECTYPE_HLST:
-      rec->r.hlst.next = buf32_to_ulong(p); p += 4;
+      rec->r.hlst.next = buf32_to_ulong(p);
+      p += 4;
       for (i=0; i < ITEMS_PER_HLST_RECORD; i++)
         {
-          rec->r.hlst.rnum[i] = buf32_to_ulong(p); p += 4;
+          rec->r.hlst.rnum[i] = buf32_to_ulong(p);
+          p += 4;
 	}
       break;
 
     case RECTYPE_TRUST:
-      memcpy (rec->r.trust.fingerprint, p, 20); p+=20;
+      memcpy (rec->r.trust.fingerprint, p, 20);
+      p+=20;
       rec->r.trust.ownertrust = *p++;
       rec->r.trust.depth = *p++;
       rec->r.trust.min_ownertrust = *p++;
       p++;
-      rec->r.trust.validlist = buf32_to_ulong(p); p += 4;
+      rec->r.trust.validlist = buf32_to_ulong(p);
       break;
 
     case RECTYPE_VALID:
-      memcpy (rec->r.valid.namehash, p, 20); p+=20;
+      memcpy (rec->r.valid.namehash, p, 20);
+      p+=20;
       rec->r.valid.validity = *p++;
-      rec->r.valid.next = buf32_to_ulong(p); p += 4;
+      rec->r.valid.next = buf32_to_ulong(p);
+      p += 4;
       rec->r.valid.full_count = *p++;
       rec->r.valid.marginal_count = *p++;
       break;
@@ -1755,7 +1769,7 @@ tdbio_new_recnum ()
       if (offset == (off_t)(-1))
         log_fatal ("trustdb: lseek to end failed: %s\n", strerror (errno));
       recnum = offset / TRUST_RECORD_LEN;
-      assert (recnum); /* this is will never be the first record */
+      log_assert (recnum); /* this is will never be the first record */
       /* We must write a record, so that the next call to this
        * function returns another recnum.  */
       memset (&rec, 0, sizeof rec);
